#define tpc_cxx
#define PI 3.1415927
#include <iostream>
#include <fstream>

#include "TString.h"
#include "TRint.h"
#include "TTree.h"
#include "TFile.h"
#include "TCutG.h"
#include "TChain.h"
#include "TH1F.h"
#include "TH2.h"
#include "TF1.h"
#include "TH3.h"
#include "TH3F.h"
#include "TF3.h"
#include "TMarker3DBox.h"
#include "TStyle.h"
#include "TCanvas.h"
#include "TMath.h"
#include "TGraph.h"
#include "TGraph2D.h"
#include "TRandom2.h"
#include "TStyle.h"
#include "TF2.h"
#include "TH1.h"
#include "TMinuit.h"
#include "TVirtualFitter.h"
#include "TFitter.h"
#include "TPolyLine3D.h"
#include "Math/Vector3D.h"
#include "/Users/acorsi/codes/MINOS_simulation/lib/ExN03DataoRings.hh"
#include "/Users/acorsi/codes/MINOS_simulation/lib/ExN03Datai.hh"
#include "/Users/acorsi/codes/MINOS_simulation/lib/ExN03BeamIn.hh"

using namespace ROOT::Math;

double pi = PI;
struct data_t
{
	double energy1;
	double theta1;
	double phi1;
	double energy2;
	double theta2;
	double phi2;
	double x0;
	double y0;
	double z0;
	double distance;
	double energy1Rec;
	double theta1Rec;
	double phi1Rec;
	double energy2Rec;
	double theta2Rec;
	double phi2Rec;
	double x0Rec;
	double y0Rec;
	double z0Rec;
	bool detection1;
	bool detection2;
};
data_t data;

 ExN03DataoRings *datao = new ExN03DataoRings();
 ExN03BeamIn *beamIn = new ExN03BeamIn();
 
// define the parameteric line equation 
void ClearEvent()
{
	data.energy1=0.;
	data.theta1=0.;
	data.phi1=0.;
	data.energy2=0.;
	data.theta2=0.;
	data.phi2=0.;
	data.x0=0.;
	data.y0=0.;
	data.z0=0.;
	data.distance=0.;
	data.energy1Rec=0.;
	data.theta1Rec=0.;
	data.phi1Rec=0.;
	data.energy2Rec=0.;
	data.theta2Rec=0.;
	data.phi2Rec=0.;
	data.x0Rec=0.;
	data.y0Rec=0.;
	data.z0Rec=0.;
	data.detection1=false;
	data.detection2=false;	
}

void DiscriminateWhichProton(int &pr, double x0, double y0, double z0, double v01, double theta1, double phi1, double v02, double theta2, double phi2, double x, double y, double z)
{
   	double m = 1.673*pow(10.,-27);
   	double Q = 1.602*pow(10.,-19);
   	double B = 4.;
   	double R1 = m*v01*sin(theta1)/(Q*B)*1000.;
   	double R2 = m*v02*sin(theta2)/(Q*B)*1000.;
  	double omega = Q*B/m;
   	double xC1 = x0 + R1*cos(phi1 - TMath::Pi()/2.);
   	double yC1 = y0 + R1*sin(phi1 - TMath::Pi()/2.);
   	double xth1 = (xC1 + R1*cos(phi1+TMath::Pi()/2.-omega*((z-z0)/(v01*1000.*cos(theta1)))));
   	double yth1 = (yC1 + R1*sin(phi1+TMath::Pi()/2.-omega*((z-z0)/(v01*1000.*cos(theta1)))));
   	double xC2 = x0 + R2*cos(phi2 - TMath::Pi()/2.);
   	double yC2 = y0 + R2*sin(phi2 - TMath::Pi()/2.);
   	double xth2 = (xC2 + R2*cos(phi2+TMath::Pi()/2.-omega*((z-z0)/(v02*1000.*cos(theta2)))));
   	double yth2 = (yC2 + R2*sin(phi2+TMath::Pi()/2.-omega*((z-z0)/(v02*1000.*cos(theta2)))));
	
	double val[2] = {sqrt(pow(x-xth1,2)+pow(y-yth1,2)), sqrt(pow(x-xth2,2)+pow(y-yth2,2))};
	if(TMath::LocMin(2, val) == 0) pr = 1;
	if(TMath::LocMin(2, val) == 1) pr = 2;
}

double distanceBfield(double x,double y,double z, double *p) { 
   double m = 1.673*pow(10.,-27);
   double Q = 1.602*pow(10.,-19);
   double B = 4.;
   double R = m*p[3]*sin(p[4])/(Q*B)*1000.;
   double omega = Q*B/m;
   double xC = p[0] + R*cos(p[5] - TMath::Pi()/2.);
   double yC = p[1] + R*sin(p[5] - TMath::Pi()/2.);
   double dx = x - (xC + R*cos(p[5]+TMath::Pi()/2.-omega*((z-p[2])/(p[3]*1000.*cos(p[4])))));
   double dy = y - (yC + R*sin(p[5]+TMath::Pi()/2.-omega*((z-p[2])/(p[3]*1000.*cos(p[4])))));
   double d = dx*dx + dy*dy;
   return d; 
}
void SumDistanceBfield(int &, double *, double & sum, double * par, int ) { 
    
   double par1[6] = {par[0], par[1], par[2], par[3], par[4], par[5]};
   double par2[6] = {par[0], par[1], par[2], par[6], par[7], par[8]};
   
   int npoints = datao->x_pad.size(); 
   int nused=0;
   sum = 0;
   double factor=0.;
   for (int i  = 0; i < npoints; i++) 
   { 
   		factor=(double)(datao->q_pad[i])/(double)(datao->Et_pad);
   		int pr=0;
   		double d=0.;
   		DiscriminateWhichProton(pr, par[0], par[1], par[2], par[3], par[4], par[5], par[6], par[7], par[8], datao->x_pad[i], datao->y_pad[i], datao->t_pad[i]*datao->driftV);
   		if(pr==1) d = factor*distanceBfield(datao->x_pad[i],datao->y_pad[i],datao->t_pad[i]*datao->driftV,par1); 
   		if(pr==2) d = factor*distanceBfield(datao->x_pad[i],datao->y_pad[i],datao->t_pad[i]*datao->driftV,par2); 
      		sum += d;
      		nused++;
   }
   sum/=nused;
}


void SumDistanceBfield1P(int &, double *, double & sum, double * par, int ) { 
    
   double par1[6] = {par[0], par[1], par[2], par[3], par[4], par[5]};
   
   int npoints = datao->x_pad.size(); 
   int nused=0;
   sum = 0;
   double factor=0.;
   for (int i  = 0; i < npoints; i++) 
   { 
   		factor=(double)(datao->q_pad[i])/(double)(datao->Et_pad);
   		double d=0.;
   		d = factor*distanceBfield(datao->x_pad[i],datao->y_pad[i],datao->t_pad[i]*datao->driftV,par1); 
      		sum += d;
      		nused++;
   }
   sum/=nused;
}



void Select(bool &event, bool &detection)
{
	int nbP = 0;
	for(unsigned int i=0; i<datao->datai.A.size(); i++)
	{
		if((int) datao->datai.Z[i] == 1 && datao->datai.A[i] == 1 && datao->datai.detection[i] == true) {nbP += 1; detection=true;}
	}
	if(nbP>1) event = true;
	else if (nbP==1) event = false;	
}



void tpcFit(char s1[], char s2[])
{
	double Em = 938.272;
	double c = 299792458.;
	TFile *fileo = new TFile(("/home/irfulx168/mnt/acorsi/MINOS_Bfield/Analysis/result/SAMURAI/simu_Li11_250MeV_20mrad_before_random.root"), "RECREATE");
 	TTree *treeo = new TTree("TREE_RESOL", "tree_resol");
 	treeo->Branch("RESOL", &data.energy1, "data.energy1/D:data.theta1:data.phi1:data.energy2:data.theta2:data.phi2:data.x0:data.y0:data.z0:data.distance:data.energy1Rec:data.theta1Rec:data.phi1Rec:data.energy2Rec:data.theta2Rec:data.phi2Rec:data.x0Rec:data.y0Rec:data.z0Rec:data.detection1/O:data.detection2"); 
  
 	TChain *chain = new TChain("tpc","");
 	chain->Add(("/home/irfulx168/mnt/acorsi/MINOS_Bfield/Drift/result/SAMURAI/simu_Li11_250MeV_20mrad_before_random.root/MyTreeo"));
	chain->SetBranchAddress("tpco",&datao);
  		
 
 	Long64_t nentries = chain->GetEntries();
 	int nevent =0;
  	//nentries=1000;
	for (Long64_t jentry=0; jentry<nentries;  jentry++) 
	{
 		nevent++;
 		bool eventP2P=false, detection = false;
    		Long64_t LocalEntry = chain->LoadTree(jentry);
    		ClearEvent();
    		if (LocalEntry < 0) break;
    
    		chain->GetEntry(jentry);  
      		
		Select(eventP2P, detection); // selection of events where 2p are detected
		//selection des evenements P2P ou les 2P sont détectés sur les pads (c'est une affaire de seuil...)
		bool proton1 = false, proton2 = false;
		int k=0;
		for(unsigned int i=0; i<datao->datai.A.size(); i++)
		{
			if((int) datao->datai.Z[i] == 1 && datao->datai.A[i] == 1) 
			{
				data.x0 = datao->datai.x0[i]; 
				data.y0 = datao->datai.y0[i]; 
				data.z0 = datao->datai.z0[i]; 
				if(k==0)
				{
					data.energy1 = datao->datai.energy0[i];
					data.theta1 = datao->datai.theta0[i];
					data.phi1 = datao->datai.phi0[i];
				} 
				if(k==1)
				{
					data.energy2 = datao->datai.energy0[i];
					data.theta2 = datao->datai.theta0[i];
					data.phi2 = datao->datai.phi0[i];
				} 
				k++;
			}
		}
   		double Etot1 = sqrt(data.energy1*data.energy1 + 2.*data.energy1*Em + Em*Em);
   		double v0overC1 = sqrt(1.-Em*Em/Etot1/Etot1);
   		double Etot2 = sqrt(data.energy2*data.energy2 + 2.*data.energy2*Em + Em*Em);
   		double v0overC2 = sqrt(1.-Em*Em/Etot2/Etot2);
   		cerr << "x0=" << data.x0 << " y0=" << data.y0 << " z0=" << data.z0 << " v01=" << v0overC1*c << " theta1=" << data.theta1 << " phi1=" << data.phi1 <<  " v02=" << v0overC2*c << " theta2=" << data.theta2 << " phi2=" << data.phi2 << endl;
		if(datao->x_pad.size()>0)
		{
 			for (Int_t i=0;i<(int)(datao->x_pad.size());i++) 
 			{
      				int pr=0;
      				DiscriminateWhichProton(pr, data.x0, data.y0, data.z0, v0overC1*c, data.theta1, data.phi1, v0overC2*c, data.theta2, data.phi2, datao->x_pad[i], datao->y_pad[i], datao->t_pad[i]*datao->driftV);
      				if(pr==1) {proton1=true; data.detection1=true;}
      				if(pr==2) {proton2=true; data.detection2=true;}
      				if(proton1 && proton2) i = datao->x_pad.size();
      			}
      		}
		cerr << "EventP2P..." << eventP2P << "    proton1..." << proton1 << "     proton2..." << proton2 << endl;
		
		if(eventP2P && proton1 && proton2)
		{

		// generate graphs with the 3d points
			Double_t parFit[9],err[9];

			if(datao->x_pad.size()>0)
			{
   				TMinuit *min = new TMinuit(9);
   				min->SetFCN(SumDistanceBfield);

   				Double_t arglist[10];
   				arglist[0] = 3;
   				Int_t iflag; 
   				min->mnparm(0,"x0",data.x0,0.1,-100,100,iflag);
   				min->mnparm(1,"y0",data.y0,0.1,-100,100,iflag);
   				min->mnparm(2,"z0",data.z0,0.1,-0,150,iflag);
   				min->mnparm(3,"v01",v0overC1*3*pow(10.,8),0.1,0,300000000,iflag);
   				min->mnparm(4,"theta1",data.theta1*TMath::Pi()/180.,0.1,(data.theta1-10.)*TMath::Pi()/180.,(data.theta1+10.)*TMath::Pi()/180.,iflag);
   				min->mnparm(5,"phi1",data.phi1*TMath::Pi()/180.,0.1,(data.phi1-30.)*TMath::Pi()/180.,(data.phi1+30.)*TMath::Pi()/180.,iflag);
   				min->mnparm(6,"v02",v0overC2*3*pow(10.,8),0.1,0,300000000,iflag);
   				min->mnparm(7,"theta2",data.theta2*TMath::Pi()/180.,0.1,(data.theta2-10.)*TMath::Pi()/180.,(data.theta2+10.)*TMath::Pi()/180.,iflag);
   				min->mnparm(8,"phi2",data.phi2*TMath::Pi()/180.,0.1,(data.phi2-30.)*TMath::Pi()/180.,(data.phi2+30.)*TMath::Pi()/180.,iflag);
    

   				arglist[0] = -1; // no printing    
   				//min->mnexcm("SET PRINTOUT",arglist,1,iflag);
   				arglist[0] = 10000; // number of function calls 
   				arglist[1] = 0.0001; // tolerance 
   				min->mnexcm("MIGRAD",arglist,2,iflag);

   				int nvpar,nparx; 
   				double amin,edm, errdef;
   				min->mnstat(amin,edm,errdef,nvpar,nparx,iflag);

   		// get fit parameters
   				for (Int_t i = 0; i <9; i++)
   				{
   					min->GetParameter(i,parFit[i],err[i]);
    					cout<<i<<" "<<parFit[i]<< "    " << err[i] << endl;
 				}
				
				data.energy1Rec = sqrt(Em*Em/(1-parFit[3]*parFit[3]/c/c))-Em;
				data.theta1Rec = parFit[4]*180./TMath::Pi();
				data.phi1Rec = parFit[5]*180./TMath::Pi();
				data.x0Rec = parFit[0];
				data.y0Rec = parFit[1];
				data.z0Rec = parFit[2];
				data.energy2Rec = sqrt(Em*Em/(1-parFit[6]*parFit[6]/c/c))-Em;
				data.theta2Rec = parFit[7]*180./TMath::Pi();
				data.phi2Rec = parFit[8]*180./TMath::Pi();
				delete min;
			}
	
   		cerr << "x0=Rec" << data.x0Rec << " y0Rec=" << data.y0Rec << " z0Rec=" << data.z0Rec << " energy1Rec=" << data.energy1Rec << " theta1Rec=" << data.theta1Rec << " phi1Rec=" << data.phi1Rec <<  " energy2Rec=" << data.energy2Rec << " theta2Rec=" << data.theta2Rec << " phi2Rec=" << data.phi2Rec << endl;
				   			   		
			if(detection)
			{
				cerr << "Filling tree..." << endl;
				treeo->Fill();
			}
		} 
		
		if(!(eventP2P && proton1 && proton2))
		{

		// generate graphs with the 3d points
			Double_t parFit[9],err[9];

			if(datao->x_pad.size()>0)
			{
   				TMinuit *min = new TMinuit(6);
   				min->SetFCN(SumDistanceBfield1P);

   				Double_t arglist[10];
   				arglist[0] = 3;
   				Int_t iflag; 
   				min->mnparm(0,"x0",data.x0,0.1,-100,100,iflag);
   				min->mnparm(1,"y0",data.y0,0.1,-100,100,iflag);
   				min->mnparm(2,"z0",data.z0,0.1,-0,150,iflag);
   				min->mnparm(3,"v01",v0overC1*3*pow(10.,8),0.1,0,300000000,iflag);
   				min->mnparm(4,"theta1",data.theta1*TMath::Pi()/180.,0.1,(data.theta1-10.)*TMath::Pi()/180.,(data.theta1+10.)*TMath::Pi()/180.,iflag);
   				min->mnparm(5,"phi1",data.phi1*TMath::Pi()/180.,0.1,(data.phi1-30.)*TMath::Pi()/180.,(data.phi1+30.)*TMath::Pi()/180.,iflag);
    

   				arglist[0] = -1; // no printing    
   				//min->mnexcm("SET PRINTOUT",arglist,1,iflag);
   				arglist[0] = 10000; // number of function calls 
   				arglist[1] = 0.0001; // tolerance 
   				min->mnexcm("MIGRAD",arglist,2,iflag);

   				int nvpar,nparx; 
   				double amin,edm, errdef;
   				min->mnstat(amin,edm,errdef,nvpar,nparx,iflag);

   		// get fit parameters
   				for (Int_t i = 0; i <6; i++)
   				{
   					min->GetParameter(i,parFit[i],err[i]);
    					cout<<i<<" "<<parFit[i]<< "    " << err[i] << endl;
 				}
				
				data.energy1Rec = sqrt(Em*Em/(1-parFit[3]*parFit[3]/c/c))-Em;
				data.theta1Rec = parFit[4]*180./TMath::Pi();
				data.phi1Rec = parFit[5]*180./TMath::Pi();
				data.x0Rec = parFit[0];
				data.y0Rec = parFit[1];
				data.z0Rec = parFit[2];
				delete min;
			}
	
   		cerr << "x0=Rec" << data.x0Rec << " y0Rec=" << data.y0Rec << " z0Rec=" << data.z0Rec << " energy1Rec=" << data.energy1Rec << " theta1Rec=" << data.theta1Rec << " phi1Rec=" << data.phi1Rec <<  " energy2Rec=" << data.energy2Rec << " theta2Rec=" << data.theta2Rec << " phi2Rec=" << data.phi2Rec << endl;
				   			   		
			if(detection)
			{
				cerr << "Filling tree..." << endl;
				treeo->Fill();
			}
		} 
		
	}//loop on entries

fileo->Write();
fileo->Close();
}

int main(int argc, char** argv) { 
   
   TRint *theApp = new TRint("Rint",&argc,argv,0,0);   
   tpcFit("","");
   theApp->Run(kTRUE);
   delete theApp;
   return 0;
}

